(function(_window) {
    'use strict';

    /**
     * Regex replacer for HTML escaping.
     *
     * @param {string} match matched character (must exist in HTML_CHARS).
     * @return {string} Html entity
     */
    function htmlReplacer(match) {
        return HTML_CHARS[match];
    }

    var uniqIdCounter = 0,
        HTML_CHARS = {
            '&': '&amp;',
            '<': '&lt;',
            '>': '&gt;',
            '"': '&quot;',
            "'": '&#x27;',
            '/': '&#x2F;',
            '`': '&#x60;'
        },
        utils = {
            /**
             * Get a unique id.
             *
             * @param {string} prefix
             */
            uniqueId: function(prefix) {
                var uniqId = ++uniqIdCounter;

                // Simply incrementing (as lodash does) won't work if we want to preserve id's
                // when we rebind (open save draft).
                return prefix + '-' + Math.ceil(Math.random() * 1000000) + '-' + uniqId;
            },

            /**
             * Return true if the value is an Integer.
             *
             * Uses ES6 function if defined.
             *
             * @param {*} val
             */
            isInteger: Number.isInteger || function(val) {
                return typeof val === "number" &&
                       isFinite(val) &&
                       Math.floor(val) === val;
            },

            /**
             * Functional utility to loop through an array calling a
             * callback each time to test for a condition to be meet, and
             * exit on first find.
             *
             * @param {Array} arr
             * @param {Function} cb
             * @returns {Boolean}
             */
            findFirst: function(arr, cb) {
                var i;

                for(i=0; i < arr.length; i++) {
                    if (cb(arr[i])) {
                        return true;
                    }
                }

                return false;
            },

            /**
             * For each capable of handling HtmlCollections and NodeLists.
             *
             * Note: use {@link Array#forEach} if you can.  This method is only useful
             * for collections that do not contain the forEach method on their prototype.
             *
             * @param {HtmlCollection|NodeList} collection
             * @param {function} cb
             */
            forEach: function (collection, cb) {
                for (var i = 0; i < collection.length; i++) {
                    /**
                     * @callback cb
                     * @param {*} value
                     * @param {*} index
                     */
                    cb.call(this, collection[i], i);
                }
            },


            /**
             * Get the value of a key within an object.
             * Path to the value is specified by an array of keys.
             * If value is not found, defaultValue will be returned.
             *
             * @param {object} o
             * @param {array} path
             * @param {*} defaultValue
             *
             * @return {*} value at path if found, else default value if provided, else null
             */
            get: function(o, path, defaultValue) {
                // TODO check if o is an object
                var i,
                    p = Array.isArray(path) ? path : [], // TODO allow conversion from 'array-like' types and from dot-notation string
                    l = p.length;

                for (i = 0; o != null && i < l; i++) {
                    o = o[p[i]];
                }

                return o != null ? o : defaultValue;
            },

            // Port from underscore
            debounce: function(func, wait, immediate) {
                var timeout;
                return function() {
                    var context = this, args = arguments;
                    var later = function() {
                        timeout = null;
                        if (!immediate) {
                            func.apply(context, args);
                        }
                    };
                    var callNow = immediate && !timeout;
                    clearTimeout(timeout);
                    timeout = setTimeout(later, wait);
                    if (callNow) {
                        func.apply(context, args);
                    }
                };
            },

            /**
             * Escape Html string
             *
             * String escaping meets OWASP recommendations.
             * This method is taken from YUI Escape API.
             * {@see http://yuilibrary.com/yui/docs/api/files/escape_js_escape.js.html}
             *
             * @param {string} html string
             * @return {string} escaped html string
             */
            escapeHtml: function(string) {
                return (string + '').replace(/[&<>"'\/`]/g, htmlReplacer);
            },

            noop: function() { },

            /**
             * Fire a callback as soon as the DOMContentLoaded has been fired.
             *
             * Function is executed immediately if called after DOMContentLoaded
             * has already been fired.
             *
             * @param {function} cb callback function to execute after DOMContentLoaded
             */
            onDomContentLoaded: function(cb) {
                if (document.readyState === 'interactive' || document.readyState === 'complete') {
                    cb.call(this);
                } else {
                    document.addEventListener('DOMContentLoaded', function wrapper() {
                        cb.call(this);
                        document.removeEventListener('DOMContentLoaded', wrapper);
                    });
                }
            },

            /**
             * Remove an element from the DOM
             */
            removeElement: function(el) {
                if (el) {
                    if (!('remove' in Element.prototype)) {
                        if (el.parentNode) {
                            el.parentNode.removeChild(el);
                        }
                    } else {
                        el.remove();
                    }
                }
            },

            /**
             * Check if given element is descendant of a parent element
             */
            isDescendant: function(parent, child) {
                var node = child.parentNode;
                while (node != null) {
                    if (node == parent) {
                        return true;
                    }
                    node = node.parentNode;
                }
                return false;
            },

            /**
             * Check if given element is descendant of a given tag
             */
            isDescendantOfTag: function(parentTag, child) {
                var node = child.parentNode;
                while (node != null) {
                    if (node.tagName == parentTag) {
                        return true;
                    }
                    node = node.parentNode;
                }
                return false;
            }
        };

    _window.utils = utils;
})(window);
