(function(ViewController) {
    'use strict';

    /**
     * @param {number} maxWidth - max width the image is not allowed to exceed
     */
    function ExifImageViewController(maxWidth) {
        ViewController.apply(this, arguments);

        this.maxWidth = maxWidth;
    }

    ExifImageViewController.prototype = Object.create(ViewController.prototype);
    ExifImageViewController.prototype.constructor = ExifImageViewController;

    function rotate(imgEl, degrees, shouldFlip, handleSizeChange) {
        /*
         * Note this does not support arbitrary setting of degrees
         * These must match an existing class in the CSS.
         *
         * Alternatively, we could support any degree by setting an
         * inline style, but this is not prefered for browser performance.
         */
        if (degrees) {
            imgEl.classList.add('rotate-' + degrees);
        }
        if (shouldFlip) {
            imgEl.classList.add('flip');
        }

        /**
         * @param {Node} imgEl
         * @param {number} rotation - rotation in degrees
         * @param {function} handleSizeChange
         */
        function setDimensions(imgEl, rotation) {
            // Box model is unaffected by css transformations.
            // We need to handle the change in height manually.
            var originalWidth = imgEl.naturalWidth,
                originalHeight = imgEl.naturalHeight,
                sizeCorrectingContainerEl,
                newHeight;

            if (rotation % 180 === 0) {
                imgEl.style.maxWidth = this.maxWidth + 'px';
                imgEl.style.maxHeight = 'none';
            } else {
                /*
                 * Height is actually the width
                 *
                 * The height of the image will become the width (and visa versa) after rotating.
                 * These bounds will be correct for displaying the rotated image.  However, css transform
                 * does not affect the flow of the content surrounding the tranformed elements.  This causes
                 * undesired side affects since the DOM still uses the width (actually the image height now)
                 * for its bounding box when doing reflows.
                 *
                 * To get around this, we use a container element with the height and width of the image after
                 * rotation.  Finally, we set the overflow to hidden causing the browser to disregard the image
                 * element width/height when reflowing.
                 */
                newHeight = this.maxWidth < originalHeight ?  this.maxWidth : originalWidth;
                imgEl.style.height = newHeight + 'px';
                imgEl.style.width = ((originalWidth / originalHeight) * newHeight) + 'px';

                sizeCorrectingContainerEl = document.createElement('div');
                sizeCorrectingContainerEl.style.overflow = 'hidden';
                sizeCorrectingContainerEl.style.height = imgEl.style.width;
                sizeCorrectingContainerEl.style.width = imgEl.style.height;

                // replace the current img element with new wrapped element
                imgEl.parentNode.replaceChild(sizeCorrectingContainerEl, imgEl);

                // insert the img into the newly added container
                sizeCorrectingContainerEl.appendChild(imgEl);
            }

            // make sure we resize the WebView since rotating the image can change the body height
            handleSizeChange();
        }

        if (imgEl.naturalWidth !== undefined && imgEl.naturalWidth !== 0) {
            // image already loaded
            setDimensions.call(this, imgEl, degrees);
        } else {
            // wait for image to load
            imgEl.addEventListener('load', setDimensions.bind(this, imgEl, degrees));
        }
    }

    /**
     * @param {HtmlElement} imgEl
     * @param {object} exif
     * @param {number} exif.orientation
     * @param {function} handleSizeChange
     */
    ExifImageViewController.prototype.attach = function(imgNode, exif, handleSizeChange) {
        // rotate image
        switch(exif.orientation) {
            // Rotate
            case 2:
                // just flip
                rotate.call(this, imgNode, false, true, handleSizeChange);
                break;
            case 6:
                rotate.call(this, imgNode, 90, false, handleSizeChange);
                break;
            case 5:
                rotate.call(this, imgNode, 90, true, handleSizeChange);
                break;
            case 3:
                rotate.call(this, imgNode, 180, false, handleSizeChange);
                break;
            case 4:
                rotate.call(this, imgNode, 180, true, handleSizeChange);
                break;
            case 8:
                rotate.call(this, imgNode, 270, false, handleSizeChange);
                break;
            case 7:
                rotate.call(this, imgNode, 270, true, handleSizeChange);
                break;
            default:
                // By default set the max width to window max width
                rotate.call(this, imgNode, false, false, handleSizeChange);
        }
    };

    window.ExifImageViewController = ExifImageViewController;
})(window.ViewController);
