/* global ConversationInterface */
(function(_strings, ImagesController, _utils, ExifImageViewController){
    'use strict';

    /** @var {String} class used by the backend to mark Show More content **/
    var SHOW_MORE_CLASSNAME = 'yQTDBase',

        bodyClickHandler = null,

        /**
          * @var {Boolean} true if the user has taken an action on the message
          *                that would cause the height to be modified
          */
        hasUserAlteredSize = false,

        /** @var {HTMLElement} the Show More/Less button **/
        showMoreBtn,

        /** @var {Number} the height of the collapsed content **/
        collapsedHeight,

        /** @var {boolean} a flag tracking animation in progress **/
        isAnimating,

        /** Resize with debounce **/
        resize,

        /** @var {function} callback function to be executed after the animation is complete **/
        afterAnimationCallback,

        /** @var {ImagesController} controller handling all images in the document **/
        imageController = new ImagesController();

    /**
     * Pass height to Java to set the viewHolder size and attach click event handler
     */
    function init() {
        window.onresize = function() {
            if (!isAnimating) {
                resize();
            }
        };

        attachImageListeners(document.body, function(event) {
            if (event && event.target) {
                ConversationInterface.handleImageClick(event.target.src);
            }
        },
        function() {
            if (!isAnimating) {
                // resize the WebView now that we got a new image
                resize();
            } else {
                // currently animating, so wait until after the animation to fix the height
                afterAnimationCallback = function() {
                    resize();
                };
            }
        });

        /**
          * Creates a the Show More button with styles, onclick handler, and then
          * appends it to the body element.
          */
        function addShowMoreButton() {
            var textNode = document.createTextNode(_strings.mailsdk_show_more);

            showMoreBtn = document.createElement('button');

            showMoreBtn.dataset.isShowMore = 'true';
            showMoreBtn.appendChild(textNode);

            /*
             * Click handler for the Show More/Less button.
             *
             * Responsible for setting up the content and calling the Java method to start the
             * animation.
             */
            showMoreBtn.onclick = function() {
                // User action will modify the size.
                hasUserAlteredSize = true;

                // if the button is currently showing 'Show more' then we need to expand
                var isExpanding = showMoreBtn.dataset.isShowMore === 'true';

                if (isExpanding) {
                    // When expanding, we want the content to already be visible before the
                    // animation begins.
                    collapsedHeight = getHeight();
                    document.body.classList.add('show-more');
                }

                toggleShowMore();

                isAnimating = true;

                window.formatMessage();

                // Starts the expand/collapse animation
                ConversationInterface.animateShowMore(getWidth(), isExpanding ? document.body.scrollHeight : collapsedHeight);
            };

            // Style the Button
            showMoreBtn.style.backgroundColor = '#F1F1F5';
            showMoreBtn.style.color = '#878C91';
            showMoreBtn.style.borderRadius = '2px';
            showMoreBtn.style.padding = '6px';

            // Reset default button styles
            showMoreBtn.style.border = 'none';
            showMoreBtn.style.appearance = 'none';
            showMoreBtn.style.outline = 'none';

            document.body.appendChild(showMoreBtn);
        }

        if (document.getElementsByClassName(SHOW_MORE_CLASSNAME).length) {
            // Show More content is available.  So, add the Show More button to the DOM.
            addShowMoreButton();
        }

        resize();

        if (!bodyClickHandler) {
            bodyClickHandler = document.body.addEventListener('click', bodyClick);
        }
    }

    /**
     * Resize the WebView to fit the contents.
     */
    function _resize() {
        ConversationInterface.onSizeChanged(getWidth(), getHeight(), hasUserAlteredSize);
        document.body.style.opacity = 1;
    }

    resize = _utils.debounce(_resize, 100, false);

    /**
     * Route all body clicks to Java interface
     */
    function bodyClick() {
        ConversationInterface.bodyClick();
    }

    /**
     * Get current width
     */
    function getWidth() {
        return window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth;
    }

    /**
     * Get current height
     *
     * To get an accurate height value we check a combination of attributes on the body and html attribute.
     *
     * Note that we used to also check the document.body.scrollHeight, but Chromium has an open bug for inaccurate
     * body.scrollHeight.  Can track the bug here: https://code.google.com/p/chromium/issues/detail?id=34224 and
     * revist adding a check for this attribute in the future, but should check both Webkit and Blink browsers.
     *
     * Also, removed document.documentElement.clientHeight because of observed inaccuracies.
     */
    function getHeight() {
        return Math.max(
            document.documentElement.scrollHeight,
            document.documentElement.offsetHeight
        );
    }

    /**
      * Toggle the Show More/Less button's text based on the current text.
      */
    function toggleShowMore() {
        var isShowMore = (showMoreBtn.dataset.isShowMore === 'true');

        showMoreBtn.dataset.isShowMore = !isShowMore;
        showMoreBtn.firstChild.data = !isShowMore ? _strings.mailsdk_show_more : _strings.mailsdk_show_less;
    }

    /**
      * Attaches a callback function to each img tag inside a element's DOM tree that will
      * be called each time one of the images is loaded.
      *
      * @param {HTMLElement} el element containing img tags to bind callback to
      * @param {function} imgClickedCallback function to call each time an image is clicked
      * @param {function} imgLoadedCallback function to call each time an image is received
      */
    function attachImageListeners(el, imgClickedCallback, imgLoadedCallback) {
        var imgs = el.getElementsByTagName('img'),
            i;

        for(i = 0; i < imgs.length; i++) {
            imgs[i].addEventListener('click', imgClickedCallback);
            imgs[i].addEventListener('load', imgLoadedCallback);
        }
    }

    /**
      * Called from Java after the animation is finished.
      */
    window.onAnimationEnd = function() {
        // if the button is currently showing 'Show more' then we just collapsed
        var didCollapse = (showMoreBtn.dataset.isShowMore === 'true');

        isAnimating = false;

        if (didCollapse) {
            document.body.classList.remove('show-more');
        }

        if (afterAnimationCallback) {
            // Resize the WebView now that the animation is finished, to fix the height because a
            // new image came in while the animation was happening.
            afterAnimationCallback();
            afterAnimationCallback = null;
        }
    };

    document.addEventListener('DOMContentLoaded', function() {
        // Initalize the Web View's content and calculate the height of the WebView
        init();
        window.JavascriptDocumentEventHandler.handleDOMContentLoaded();
    });

    imageController.attachAllImages();

    window.refreshDuplicateImages = function() {
        if (!imageController.isEveryImageSourceUnique()) {
            imageController.retryAll();
        }
    };

    window.removeBlockedImageBoxes = function() {
        document.body.classList.remove('image-boxes');
    }

    window.addBlockedImageBoxes = function() {
        document.body.classList.add('image-boxes');
    }

    // Expose init function to be called externally, when all element  is ready
    window.calculateHeight = resize;

    /**
     * Handle Rotation for JPEG's with Exif Rotation Header.
     *
     * {@see http://sylvana.net/jpegcrop/exif_orientation.html}
     *
     * @param {string} src - url of the image
     * @param {number} orientationNumber - number corresponding to the exif header's orientation value
     */
    window.handleExifOrientation = function(src, orientationNumber) {
        // find image elements associated with the url
        var imgNodes = document.querySelectorAll('[src="' + src + '"]'),
            imgNode,
            i;

        for (i = 0; i < imgNodes.length; i++) {
            imgNode = imgNodes.item(i);
            (new ExifImageViewController(getWidth()))
                .attach(imgNode, {
                    orientation: orientationNumber
                }, resize);
        }
    };
})(window.strings, window.ImagesController, window.utils, window.ExifImageViewController);
